////////////////////////////////////////////////////////////////////////////////
// MAIN.C - HexView - User Area File Viewer
////////////////////////////////////////////////////////////////////////////////
#include <pager.h>
#include <ribbon.h>
#include <string.h>
#include "icon.h"

char VersionPtr[] = "HexView"; 
int AppStackSize = 3000;
HandleType H; FileType F;

char *files(char *s) { // Wildcard, typically: *.c, *.nbc, *.bas
  MESSAGE M; // n:Num of files, c:Item #, a:Starting item
  char buf[48]; int n=0, i=0, c=0, a=0;
  char f[99][32], z[99][32]; // 99 names of 32 chars each
  H = DbFindNext(-1, s);
  while (H >= 0) {
    strcpy(f[n], DbName(H));
    RimSprintf(buf, 48, "%d", DbSize(H,TRUE));
    strcpy(z[n++], buf);
    H=DbFindNext(H, s);
  }
  for (;;) {
    LcdRasterOp(WHITE,160,72,NULL,0,0,NULL,0,91); // Draw box
    LcdDrawBox(DRAW_BLACK,0,92,159,159);          // Draw frame
    for (i=0; i<(n<8?n:8); i++) { // (n<8?n:8) WAS 8
      LcdPutStringXY(5,94+i*8,f[a+i],-1,TEXT_NORMAL);
      LcdPutStringXY(155,94+i*8,z[a+i],-1,LCD_RIGHT_JUSTIFIED);
    }
    if (n == 0)  LcdPutStringXY(-1,94,"NO FILES",-1,LCD_CENTERED);
    LcdRasterOp(INVERT_DEST,156,8,NULL,0,0,NULL,2,94+c*8); // Hilight
    RimGetMessage(&M);                                     // Wait for key
    LcdRasterOp(INVERT_DEST,156,8,NULL,0,0,NULL,2,94+c*8); // Unhilight
    if (M.SubMsg==KEY_ESCAPE)  return ""; // ESCAPE
    switch (M.Event) {
    case THUMB_ROLL_UP: if (c>0) c--; else if (a>0) a--; 
      break;
    case THUMB_ROLL_DOWN: if (c<7 && c<n-1) c++; else if (a+c<n-1) a++;
      break;
    case THUMB_CLICK: if (n>0) return f[a+c]; else return ""; // Selected
    }
  }
}

int load(char *s, char *m) { // IN:  Name & holding memory
  int fs;                    // OUT: File size
  H=DbGetHandle(s); F=DbFileOpen(H,256);
  fs = DbSize(H,TRUE);
  DbFileRead(F, m, fs);
  DbFileClose(F,TRUE);
  return fs;
}

void HVThread(void) { /////////////////////////////////////////////////////////

  char X,Y;       // Loop variables
  char M=0;       // Display mode toggle (SPACE)
  char b[64];     // Formatting String
  char NC=0;      // Next char typed
  char TB[16]=""; // Text buffer
  char FN[32]=""; // File name
  char FM[65536]; // Holds file
  int offset = 0, onscr, fs; // Offset, OnScreen & File size
  unsigned char v;
  MESSAGE msg;
  PID pid;
  pid.Name = VersionPtr;
  pid.EnableForeground = TRUE;
  pid.Icon = NULL;
  RimSetPID(&pid);

  strcpy(FN, files("*"));
  fs = load(FN, FM);

  for (;;) {
    onscr=offset;
    LcdClearDisplay();
    for(Y=0; Y<19; Y++) {
      RimSprintf(b,63, "%04lx:", onscr);             // Convert Offset to Hex  
      if (onscr < fs)  LcdPutStringXY(0,Y*8,b,-1,0); // Display Offset
      for(X=0; X<8; X++) {
        v = FM[onscr];
        RimSprintf(b,63,M?"%c":"%02x",v);           // Chr or Hex mode
        if (onscr < fs)  { 
          LcdPutStringXY(25+X*12,Y*8,b,-1,0);       // Display hex byte
          LcdPutStringXY(121+X*5,Y*8,&v,1,LCD_RAW); // Displat chr byte
        }
        onscr++;
      }
    }
    LcdPutStringXY(0,152,TB,-1,TEXT_INVERT);           // New offset string
    LcdPutStringXY(159,152,FN,-1,LCD_RIGHT_JUSTIFIED); // File name
    LcdDrawLine(DRAW_BLACK,0,152,160,152);             // Draw line

    RimGetMessage(&msg);
    switch (msg.Device) {
      case DEVICE_KEYPAD:
        switch (msg.Event) {
          case THUMB_ROLL_UP:   // DOWN
            if(offset>0)  offset -= 8; // WAS 144 (page)
            break;
          case THUMB_ROLL_DOWN: // UP
            if(offset<65536)  offset += 8; // WAS 144 (page)
            break;
          case KEY_DOWN:
          {
            switch (msg.SubMsg) {
              case KEY_ESCAPE: // ESC
                RibbonShowRibbon();
                RimTerminateThread();
                break;
              case ' ':
                M=(M?0:1); // Mode toggle
                break;
              case 'l':    // load new file
                strcpy(FN, files("*"));
                fs = load(FN, FM); offset = 0;
                break;
              case KEY_ENTER: // Extract new offset from string
                offset=strtoul(TB,NULL,16);
                NC=0;
                TB[NC]=0;
                break;
              case KEY_BACKSPACE:
                if(NC>0) {NC-=1; TB[NC]=0;}
                break;
              default:
                TB[NC++]=msg.SubMsg;
                TB[NC]=0;
            }
            break;
          }
        }
    }
  } // End Main for loop
}

void PagerMain(void) {
  RibbonRegisterFunction(VersionPtr, &IconBMP, 0, 40, HVThread, AppStackSize);
}
